/*
 *  main.c
 *  PHIGS
 *
 *  Created by Simone Portuesi on 08/03/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */
 
 /* Universit√† Roma 3, DIA, Informatica Grafica Esercitazione del 8/3/2006 */
 
 /* Codice principale per visualizzazione modello                          */
 
 /* Codice esempi √® in file separato, tale file deve implementare due funzioni:
    void model_display(): 
                          chiamato da display() per generare il modello    
    void  keyboard_model(unsigned char key, int x, int y): 
					      chiamata da keyboard() per gestire tasti specifici 
	Le due funzioni sono dichiarate in main.h
						                                                   */ 


/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Include my files */
#include "main.h"
#include "PHIGS.h"
#include "utility.h"

/* Utility MACRO */
#define sqr(x) ((x)*(x))
#define deg2rad(deg) (M_PI*(deg)/180)

/* The name is missleading */
/* Animated transformation */
/* Used between mousemotion and moveposittion */
struct Viewpoint {
	GLfloat center[3];
	GLfloat alpha;
	GLfloat beta;
	GLfloat gamma;
	GLfloat zoom;
};
struct Viewpoint init_viewpoint = { { 0, 0, 0}, 0, 0, 0, 1}; 
struct Viewpoint viewpoint = { { 0, 0, 0}, 0, 0, 0, 1}; 


/* Mouse movement structure */
/* Used between mouse and mousemotion to track mouse movements */
enum Motion_mode { NONE=0, ROTATE, DIST,  PAN };

struct Motion { 
	enum Motion_mode mode;
	int startx;
	int starty;
};

struct Motion motion = { NONE, 0, 0 };

GLenum shading= GL_SMOOTH;

struct ViewPoints { 
	char* name;
	struct PHIGS_View view;
};

struct ViewPoints viewPoints[] = {
	{ "onepoint",   { { { 0,0,0 }, { 1,0,0 }, { 0,0,1 } }, { PERSPECTIVE, { 10,8,30 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "twopoint",   { { { 0,0,0 }, { 3,2,0 }, { 0,0,1 } }, { PERSPECTIVE, { 10,8,30 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "threepoint", { { { 0,0,0 }, { 3,2,1 }, { 0,0,1 } }, { PERSPECTIVE, { 10,8,30 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "orthox", { { { 0,0,0 }, { 1,0,0 }, { 0,0,1 } }, { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "orthoy", { { { 0,0,0 }, { 0,1,0 }, { 0,0,1 } }, { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "orthoz", { { { 0,0,0 }, { 0,0,1 }, { 0,-1,0 } }, { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "isometric", { { { 0,0,0 }, { 0.5764323161697927 , 0.5764323161697927 , 0.5764323161697927 }, { 0,0,1 } }, 
	               { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "dimetric" , { { { 0,0,0 }, { 0.3308438212522159 , 0.8788171126619648 , 0.3308438212522159 }, { 0,0,1 } }, 
	               { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "trimetric", { { { 0,0,0 }, { 0.45864955448431455 , 0.8686315144381906 , 0.1650476058606775 }, { 0,0,1 } }, 
	               { PARALLEL, { 0,0,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "central cavalier", { { { 0,0,0 }, { 0,0,1 }, { -1,-1,0 } }, 
	               { PARALLEL, { 0,-1,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "left cavalier"   , { { { 0,0,0 }, { 0,0,1 }, { -0.5,-0.8660254037844386,1 } }, 
	               { PARALLEL, { 0,-1,1 } ,{ -1, -1,  1, 1, 1, -1,} } } },
	{ "right cavalier"   , { { { 0,0,0 }, { 0,0,1 }, { -0.8660254037844386,0.5,1 } }, 
	               { PARALLEL, { 0,-1,1 } ,{ -1, -1,  1, 1, 1, -1,} } } },
	{ "cabinet", { { { 0,0,0 }, { 0,1,0 }, { 0,0,1 } }, 
	               { PARALLEL, { -0.3535533905932738,0.3535533905932738,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "xCavalier", { { { 0,0,0 }, { 0,0,1 }, { 0,-1,0 } }, { PARALLEL, { 0,-1,1 } , { -1, -1,  1, 1, 1, -1,} } } },
	{ "yCavalier", { { { 0,0,0 }, { 0,0,1 }, { -1,0,0 } }, { PARALLEL, { 0,-1,1 } , { -1, -1,  1, 1, 1, -1,} } } }
	};
	
int N_viewPoints=15;

int n_viewpoint=0;

/* Draw standard Axes */
void axes() {
	int i;

	/* Data for just the standandar X axis */
	/* Others will be permutation on coordinates */
	GLfloat axis[][3] =   {  { -1  , 0    , 0     },
							 { 0   , 0    , 0     },
						     { 1   , 0    , 0     },
						     { 0.9 , 0.1  , 0     },
						     { 0.9 , -0.1 , 0     },
							 { 1   , 0    , 0     },
						     { 0.9 , 0    , 0.1   },
						     { 0.9 , 0    ,  -0.1 },
						     { 1   , 0    ,   0   } };
							 
	GLfloat coloraxis[][3] = { { 1.0, 0.0, 0.0 },
							   { 1.0, 1.0, 1.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 },
						       { 1.0, 0.0, 0.0 } };
			
	glLineWidth(2.0);
	
	/* X axis */		
	glBegin(GL_LINE_STRIP);
	for(i=0;i<9;i++) {
		glColor3fv(coloraxis[i]);
		glVertex3fv(axis[i]);
		}
	glEnd();
	
	/* Y axis */
	glBegin(GL_LINE_STRIP);
	for(i=0;i<9;i++) {
		glColor3f(coloraxis[i][2],coloraxis[i][0],coloraxis[i][2]);
		glVertex3f(axis[i][2],axis[i][0],axis[i][1]);
		}
	glEnd();
	
	/* Z axis */
	glBegin(GL_LINE_STRIP);
	for(i=0;i<9;i++) {
		glColor3f(coloraxis[i][1],coloraxis[i][2],coloraxis[i][0]);
		glVertex3f(axis[i][1],axis[i][2],axis[i][0]);
	}
	glEnd();
}


/* Animated model move */
/* In some viewpoints it doesn't make much sense */
void moveposition() {

	/* modelview matric */
	glMatrixMode(GL_MODELVIEW);
	
	/* Translate along (x,0,-z), (0,y,0) , (-x,0,-z) */
	glTranslatef(viewpoint.center[0]-viewpoint.center[2], 
				 viewpoint.center[1],
				 viewpoint.center[2]-viewpoint.center[0]);
				 
	/* Rotate on (x,0,-z) */
	glRotatef(viewpoint.beta, cos(M_PI_4), 0, -sin(M_PI_4) );
		
	/* Rotate on (0,y,0) */
	glRotatef(viewpoint.alpha, 0, 1, 0 );

	/* Zoom scale */
	glScalef( viewpoint.zoom, viewpoint.zoom, viewpoint.zoom);
	    
}

/* Mause Click Callback */
void mouse(int button, int state, int x, int y) {
	/* A mouse button pressed */
	if(state==GLUT_DOWN) {
		/* Save position */
		motion.startx=x;
		motion.starty=y;
		
		/* Which button? */
		switch (button) {
			/* Left -> Rotate */
			case GLUT_LEFT_BUTTON:
				motion.mode=ROTATE;
				break;
			/* Middle -> zoom */
			case GLUT_MIDDLE_BUTTON:
				motion.mode=DIST;
				break;
			/* Right -> Pan */
			case GLUT_RIGHT_BUTTON:
				motion.mode=PAN;
				break;
			default:
				motion.mode=NONE;
		}
	} else { 
		motion.mode=NONE;
	}
		
}

void mousemotion(int x, int y) {
	/* Current mode? */
	switch (motion.mode) {
		
		/* Rotate, modify alfa and beta */
		case ROTATE:
			viewpoint.alpha+=(GLfloat) ((x-motion.startx) % 360);
			viewpoint.beta+=(GLfloat) ((y-motion.starty) % 360);
			break;
		/* Zoom */
		case DIST:
			viewpoint.zoom+=4.0 * ((GLfloat) (y-motion.starty))/((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			break;
		/* Pan */
		case PAN:
			viewpoint.center[0]+=4.0 * ((GLfloat) (x-motion.startx)) / ((GLfloat) glutGet(GLUT_WINDOW_WIDTH));
			/*if(glutGetModifiers()==GLUT_ACTIVE_SHIFT) { // Non funge!!!
				viewpoint.center[2]-=4.0 * ((GLfloat) (y-motion.starty)) / ((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			} else { */
			
				viewpoint.center[1]-=4.0 * ((GLfloat) (y-motion.starty)) / ((GLfloat) glutGet(GLUT_WINDOW_HEIGHT));
			
			//}
			
			break;
		default:
			break;
	}
	
	/* Save current position */
	motion.startx=x;
	motion.starty=y;
	
	/* Redisplay */
	glutPostRedisplay();
}


void display(void) {
	/* Clear Buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	
	/* View Model */
	PHIGS_View(&(viewPoints[n_viewpoint].view));
	
	/* Draw axes */
	axes();

	/* Move */
	moveposition();
	
	/* Shading */
	glShadeModel(shading);
	
	/* Draw model */
	display_model();
	
	/* Shading */
	glShadeModel(GL_SMOOTH);


	/* SwapBuffer */
	glutSwapBuffers();
}


void reshape(int w, int h) {	
	/* Difference between width and heigh */
	float diff = ((float) (w - h)) / 2;
	
	/* Display unit wiewport */
	if (diff>0) {
		glViewport(diff, 0, w-2*diff, h);
	} else {
		glViewport(0, -diff, w, h+2*diff);
	}
	
	/* Redraw this window */
	glutPostRedisplay();
}


void keyboard(unsigned char key, int x, int y) {
	
	static int fullscreen=0;
	static int old_w,old_h;
	static int old_x,old_y;
	switch(key) {
		/* Q or esc: Quit */
		case 27:
		case 'q':
		case 'Q':
			exit(0);
			break;
		/* F: Fullscreen toogle */
		case 'f':
		case 'F':
			/* Not in full screen */
			if (!fullscreen) {
				/* Save window size and position */
				old_w=glutGet(GLUT_WINDOW_WIDTH);			
				old_h=glutGet(GLUT_WINDOW_HEIGHT);
				old_x=glutGet(GLUT_WINDOW_X);
				old_y=glutGet(GLUT_WINDOW_Y);
				/* Go fullscreen */
				glutFullScreen();
				fullscreen=1;
			} else {
				/* Set window size and shape to original values */
				glutReshapeWindow(old_w, old_h);
				glutPositionWindow(old_x, old_y);
				fullscreen=0;
			}
			glutPostRedisplay();
			break;
		/* v: Type of view */
		case 'v':
		case 'V':
			n_viewpoint=( n_viewpoint + 1 ) % N_viewPoints;
			glutPostRedisplay();
			break;
		/* s: Shading Model */
		case 's':
		case 'S':
			if(shading==GL_SMOOTH) shading=GL_FLAT;
			else shading=GL_SMOOTH;
			glutPostRedisplay();
			break;
		/* o,r,c: Reset view */
		case 'o':
		case 'O':
		case 'r':
		case 'R':
		case 'c':
		case 'C':
			memcpy(&viewpoint,&init_viewpoint,sizeof(viewpoint));
			glutPostRedisplay();
			break;
		/* Call model keyboard handling */
		default:
			keyboard_model(key, x, y);
	}
	
}

int main (int argc, const char** argv) {

	/* Init Glut */
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	
	/* Main Window */
    glutInitWindowSize(800, 600);
    glutInitWindowPosition(0, 0);
	glutCreateWindow("Glutviewer");
	
	/* Set Callbacks */
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(mousemotion);
    
	/* Glut Main Loop */
	glutMainLoop();

	/* Never reached, keep compiler happy */
	return 0;
}


